// Processing sketch for Model Rocket Telemetry
// 648.ken@gmail.com
// bugled.blogspot.com
//
// Adapted from Daneil Shiffman's Learning Processing
// http://www.learningprocessing.com/

// Ensure you have a "data" folder in same directory as sketch!


import processing.serial.*;
Serial myPort;        // The serial port
int portNum = 1;      // ADJUST THIS TO THE CORRECT COM PORT
float scaleMult = 2; // multiplier, ie. 2 = 200%
boolean WRITE = false; // write to file
int xSize = 800;
int ySize = 500;
int range = 8; // acelerometer max gs
int speed = 0;

PFont f;  // Declare PFont variable
int[] time = new int[xSize]; 
int[] sensVal1 = new int[xSize]; // storing 400 readings
int[] sensVal2 = new int[xSize]; // storing 400 readings
int[] sensVal3 = new int[xSize]; // storing 400 readings
int[] sensStat1 = new int[3]; // acel (min, ave, max)
int[] sensStat2 = new int[3]; // alt (min, ave, max)
int[] sensStat3 = new int[3]; // speed (min, ave, max)
 
float acc_res = range * 2.0 / 1000.0; // acceration resolution
int chan1 = 1; // position in payload to graph
int chan2 = 2;
int intPres = 1013; // sea level hPa
int sendValue; // keystroke to send back on serial line
PrintWriter output; // for data logging
//http://forum.processing.org/#topic/25080000000433019


void setup() {
  size(xSize, ySize);
  smooth();
  f = loadFont( "AgencyFB-Reg-16.vlw" ); // Must also use [Tools -> Font Selector]
  println(Serial.list()); // shows available ports on system
  myPort = new Serial(this, Serial.list()[portNum], 57600);//57600);
  myPort.bufferUntil('\n');
  String out = "data/" + month() + "_" + day() + "_" + hour() + "_" + minute() + "_data.csv";
  if (WRITE){
    output = createWriter(out);
    output.println("ms,raw Acell,raw press, Acel(Gs), Alt(ft), Speed(mph)"); // headers
  }
  graph();
}


void draw() {
  graph();
  writeText();
}


void graph() {
  background(255);
  strokeWeight(2); 
  stroke(#FF0000); // red

  float dash[] = {5, 3, 9, 4};
  float dash2[] = {1, 10};
  float dash3[] = {1, 20};

  dashline(150, ySize * 0.125, xSize, ySize * 0.125, dash3);   
  dashline(0, ySize * 0.250, xSize, ySize * 0.250, dash);   
  dashline(0, ySize * 0.375, xSize, ySize * 0.375, dash2); 
  line(0, ySize * 0.5, xSize, 250);
  dashline(0, ySize * 0.625, xSize, ySize * 0.625, dash2); 
  dashline(0, ySize * 0.750, xSize, ySize * 0.750, dash); 
  dashline(0, ySize * 0.875, xSize, ySize * 0.875, dash3); 

  for (int x = 0; x < xSize - 1; x++){
    stroke(#0080FF); //blue
    //if(sensVal1[x] != 0){   
      line(x, height - sensVal1[x] * scaleMult - ySize / 2, x+1, height - sensVal1[x+1] * scaleMult - ySize / 2);
    //}
    stroke(#00FF00); //green
    //if(sensVal2[x] != 0){   
      line(x, height - inFeet(sensVal2[x]) - ySize / 2, x+1 , height - inFeet(sensVal2[x+1]) - ySize / 2);
    //}
  }
}


void writeText(){
  
  textFont(f,18); // Specify font to be used

  String dispText;
  fill(#0000FF);        // blue
  text ( "Acel (Gs)", 15, 20);
  dispText = "max: " + round(sensStat1[2] * acc_res, 2);
  text ( dispText, 10, 80);  
  dispText = " cur: " + round(sensVal1[xSize - 1] * acc_res, 2);
  text ( dispText, 10, 60);
  dispText = " min: " + round(sensStat1[0] * acc_res, 2);
  text ( dispText, 10, 40);


  fill(#018000);        // green
  text ( "Alt (ft)", 75, 20);
  dispText = "max: " + round(inFeet(sensStat2[2]), 1);
  text ( dispText, 75, 80);  
  dispText = " cur: " + round(inFeet(sensVal2[xSize - 1]), 1);
  text ( dispText, 75, 60);
  dispText = " min: " + round(inFeet(sensStat2[0]), 1);
  text ( dispText, 75, 40);

  /*
  fill(#FF0000);        // red
  text ( "Vel (mph)", 75 + 60, 20);
  dispText = "max: " + round(sensStat3[2], 0);
  text ( dispText, 75 + 60, 80);  
  dispText = " cur: " + round(sensVal3[xSize - 1], 0);
  text ( dispText, 75 + 60, 60);
  dispText = " min: " + round(sensStat3[0], 0);
  text ( dispText, 75 + 60, 40);
  */
  
  textFont(f,16); // Specify font to be used
  fill(#FF0000);        // Red  
  //dispText = "+" + 0.5 / scaleMult + " g";
  dispText = "+" + (125 * acc_res / scaleMult) + " g";
  text ( dispText, 1, 122);
  //dispText = "+" + 0.25 / scaleMult + " g";
  //text ( dispText, 1, 185);
  dispText = "0 g";
  text ( dispText, 1, 247);
  //dispText = "-" + 0.25 / scaleMult + " g";
  //text ( dispText, 1, 309);
  //dispText = "-" + 0.5 / scaleMult + " g";
  dispText = "-" + (125 * acc_res / scaleMult) + " g";
  text ( dispText, 1, 372);
  
}


void getRead() {// load single reading into array at pos x
  //int temp;
  //int newVal;
  //int dispOffSet; // for shifting text

  // shift all readings left, check max/min
  sensStat1[0] = height; //clear min
  sensStat1[2] = 0; //clear max
  sensStat2[0] = height; //clear min
  sensStat2[2] = 0; //clear max
  sensStat3[0] = 0; //clear min
  sensStat3[2] = 0; //clear max
  
  for(int x = 0; x < xSize - 1; x++){
    sensVal1[x] = sensVal1[x + 1];
    sensVal2[x] = sensVal2[x + 1];
    sensVal3[x] = sensVal3[x + 1];    
    time[x] = time[x + 1];

    if (sensVal1[x] < sensStat1[0]){ // new min
      sensStat1[0] = sensVal1[x]; }
    if (sensVal1[x] > sensStat1[2]){ // new max
      sensStat1[2] = sensVal1[x];}
    if (sensVal2[x] < sensStat2[0]){ // new min
      sensStat2[0] = sensVal2[x]; }
    if (sensVal2[x] > sensStat2[2]){ // new max
      sensStat2[2] = sensVal2[x];}
    if (sensVal3[x] < sensStat3[0]){ // new max
      sensStat3[0] = sensVal3[x];}
    if (sensVal3[x] > sensStat3[2]){ // new max
      sensStat3[2] = sensVal3[x];
    }
  }
}


void serialEvent (Serial myPort) {
  // get the ASCII string:
  String inString = myPort.readStringUntil('\n');
  if (inString != null) {
    //print(inString);
    int[] vals = int(splitTokens(inString, ",*")); 

    time[xSize - 1] = vals[0];
    sensVal1[xSize - 1] = vals[chan1]; 
    sensVal2[xSize - 1] = vals[chan2];

    float x1 = 0;
    float x2 = 0;
    for (int z = 1; z <= 5; z++){ // average last five readings
      x1 = x1 + sensVal2[xSize - z];
      x2 = x2 + sensVal2[xSize - z - 1];
    }
    x1 = x1 / 5.0;
    x2 = x2 / 5.0;
    int t1 = time[xSize - 1];
    int t2 = time[xSize - 1 - 1];
    if (t1 < t2) { t1 = t1 + 60000;}
    sensVal3[xSize - 1] = int((inFeet(int(x1 - x2)) / ((t1 - t2) / 1000.0))* 0.681);

    // sensVal2[xSize - 1] = int(alt); // units = Pascal (0.01 hPa), 1 ATM = 101,325 Pa
    if (vals[chan1] > 250 || vals[chan1] < -250 ){
      scaleMult = 0.5; }
    else if (vals[chan1] > 125 || vals[chan1] < -125 ){
      scaleMult = 1; }
    else if (vals[chan1] > 62 && vals[chan1] < -62 ){
      scaleMult = 1.5; }
    else { scaleMult = 2; }

    if (WRITE){    
      output.print(time[xSize - 1]);
      output.print(",");
      output.print(sensVal1[xSize - 1]); // accel
      output.print(",");
      output.print(sensVal2[xSize - 1]); // press
      output.print(",");
      output.print(round(sensVal1[xSize - 1] * acc_res, 2));
      output.print(",");
      output.print(round(inFeet(sensVal2[xSize - 1]),1));
      output.print(",");
      output.println(sensVal3[xSize - 1]);
      output.flush();
    }
    print("\n");
  }
  getRead();
}


void keyTyped() {
  int inKey = int(key);
  println("typed " + inKey + " " + keyCode);
  if (inKey == 114 || inKey == 119 || inKey == 99){
    if (inKey == 114) {myPort.write(114);} // r = read mem
    if (inKey == 119) {myPort.write(119);} // w = write mem
    if (inKey == 99) {myPort.write(99);} // c = clear mem
    if (inKey == 114 || inKey == 99) {clear();}
  }
}



float round(float val, int dp)
{ //http://processing.org/discourse/yabb2/YaBB.pl?num=1161801652
  return int(val*pow(10,dp))/pow(10,dp);
} 


/* http://processing.org/discourse/yabb2/YaBB.pl?num=1202486379
 * Draw a dashed line with given set of dashes and gap lengths. 
 * x0 starting x-coordinate of line. 
 * y0 starting y-coordinate of line. 
 * x1 ending x-coordinate of line. 
 * y1 ending y-coordinate of line. 
 * spacing array giving lengths of dashes and gaps in pixels; 
 *  an array with values {5, 3, 9, 4} will draw a line with a 
 *  5-pixel dash, 3-pixel gap, 9-pixel dash, and 4-pixel gap. 
 *  if the array has an odd number of entries, the values are 
 *  recycled, so an array of {5, 3, 2} will draw a line with a 
 *  5-pixel dash, 3-pixel gap, 2-pixel dash, 5-pixel gap, 
 *  3-pixel dash, and 2-pixel gap, then repeat. 
 */ 
void dashline(float x0, float y0, float x1, float y1, float[ ] spacing) 
{ 
  float distance = dist(x0, y0, x1, y1); 
  float [ ] xSpacing = new float[spacing.length]; 
  float [ ] ySpacing = new float[spacing.length]; 
  float drawn = 0.0;  // amount of distance drawn 
 
  if (distance > 0) 
  { 
    int i; 
    boolean drawLine = true; // alternate between dashes and gaps 
 
    /* 
      Figure out x and y distances for each of the spacing values 
      I decided to trade memory for time; I'd rather allocate 
      a few dozen bytes than have to do a calculation every time 
      I draw. 
    */ 
    for (i = 0; i < spacing.length; i++) 
    { 
      xSpacing[i] = lerp(0, (x1 - x0), spacing[i] / distance); 
      ySpacing[i] = lerp(0, (y1 - y0), spacing[i] / distance); 
    } 
 
    i = 0; 
    while (drawn < distance) 
    { 
      if (drawLine) 
      { 
        line(x0, y0, x0 + xSpacing[i], y0 + ySpacing[i]); 
      } 
      x0 += xSpacing[i]; 
      y0 += ySpacing[i]; 
      /* Add distance "drawn" by this line or gap */ 
      drawn = drawn + mag(xSpacing[i], ySpacing[i]); 
      i = (i + 1) % spacing.length;  // cycle through array 
      drawLine = !drawLine;  // switch between dash and gap 
    } 
  } 
}


void clear(){
  for(int x = 0; x < xSize - 1; x++){
    sensVal1[x] = 0;
    sensVal2[x] = 0;
    }  
  sensStat1[0] = height; //clear min
  sensStat1[2] = 0; //clear max
  sensStat2[0] = height; //clear min
  sensStat2[2] = 0; //clear max
  graph();
}


float inFeet(int val){
  return (val / 100.0) * 8.43 * 3.28; 
}


void exit(){
  if (WRITE){
    output.flush();
    output.close(); // close output file
  }
  exit();
}
